function [x,y,h] = pickdata(h)
%PICKDATA   Picks data from the active curve.
%   [X,Y] = PICKDATA returns the data (X,Y) from the active curve. If
%   several curves are present, picks the first one (see FITPARAM to
%   change this).
%
%   [X,Y] = PICKDATA(H) returns the data (X,Y) of the curve specified by
%   the handle H (eg, GCF, GCO etc). GCF, the current figure, is taken by
%   default, so that the first curve of the figure is used.
%
%   [X,Y,H] = PICKDATA(...) also returns the handle to the current curve.
%
%   See also SELECTDATA, SELECTFIT, SHOWFIT, FIT.

%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.03,  Date: 2006/07/06
%   This function is part of the EzyFit Toolbox

% History:
% 2005/05/31: v1.00, first version.
% 2006/02/23: v1.01, better check if a current figure exists.
% 2006/03/07: v1.02, also works with histogram plots.
% 2006/07/06: v1.03, pick the first or last curve, according to the
%                    variable fp.whichpickdata in fitparam.m


hf=get(0,'CurrentFigure');
if ~length(hf),
    error('EzyFit:pickdata:InvalidInput', 'No active figure.');
end;

%new v1.03
if exist('fitparam.m','file'),
    fp=fitparam;
else
    error('No fitparam file found.');
end;

if ~exist('h','var'), h=gco; end;

co=get(h); % search for active curve

% if no active curve, take the 1st curve of the active fig.
if ((~length(co)) || (~isfield(co,'XData'))),
    ha=get(gcf,'CurrentAxes');
    if length(ha),
        ca=get(ha);
        if length(ca.Children),
            if strcmp(fp.whichpickdata,'first'), % new v1.03
                h=ca.Children(end);  % (end) is the first curve
            else
                h=ca.Children(1);  % (1) is the last curve
            end;
            co=get(h); 
        else
            error('EzyFit:pickdata:InvalidInput', 'No curve in the active figure.');
        end;
    else
        error('EzyFit:pickdata:InvalidInput', 'No axis in the active figure.');
    end;
end;

x=co.XData;
y=co.YData;

%new v1.02: if the data originates from an histogram plot,
%keeps only the 2nd (or 3rd) row:
if size(x,1)~=1,
    x=x(2,:);
    y=y(2,:);
end;

if nargout==0,
    x,
    y,
    clear x y;
end;
