function fout = selectfit(varargin)
%SELECTFIT  Fit the selected data from the active curve
%   SELECTFIT displays the fit specified in a dialog box for the data
%   selected from the active curve by drawing a polygon, and repeats the
%   procedure until an empty polygon is drawn. Use the left mouse button to
%   draw the polygon. Double click or right-click to close the polygon. The
%   last trial remains displayed. Use UNDOFIT or RMFIT to remove the fit.
%   The display settings are defined in the file FITPARAM.
%
%   By default, the first curve in the active figure is used (see FITPARAM
%   to change this default behavior). To fit another curve, select it
%   before calling SELECTFIT.
%
%   SELECTFIT(FUN) specifies the fitting string FUN. FUN may be either a
%   default fit (eg, 'exp', 'power'), a user-defined fit (see EDITFIT),
%   or directly a fit equation (eg, 'c+a*exp(-x/x0)'). See FIT for the
%   syntax of FUN. FUN may also be any valid interpolation method string
%   (eg 'spline', 'cubic'... excepted 'linear'). See INTERP1 for the valid
%   interpolation methods.
%
%   SELECTFIT(F) use the equation defined in the fit structure F.
%
%   F = SELECTFIT(...) does the same, and also return the fit structure F.
%   F has the same content as the fit structure returned by FIT (see FIT
%   for details), and also contains a handle to the equation box and to the
%   curve.
%
%   Note that if the option 'lin' or 'log' is not specified in the string
%   FUN, SELECTFIT checks the Y-scale of the current figure to determine
%   whether Y or LOG(Y) has to be fitted.
%
%   Examples:
%      plotsample('power');
%      selectfit('power; log');
%
%      plotsample('poly2');
%      f = selectfit('z(v) = poly3');
%      editcoeff(f);
%
%   See also SHOWFIT, FIT, UNDOFIT, RMFIT, PLOTSAMPLE, SELECTDATA, INTERP1.

%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 2.11,  Date: 2006/09/06
%   This function is part of the EzyFit Toolbox

% History:
% 2005/10/17: v1.00, first version.
% 2005/10/18: v1.01, no display if no output arg.
% 2005/10/31: v1.10, also displays R. text is displayed in an annotation
%                    box. now uses pickdata. also accepts interpolation.
% 2005/11/03: v1.20, use the file 'fitparam.m' for the fit default values.
% 2005/11/05: v1.21, also displays the lin/log mode
% 2005/12/06: v1.30, opens a dialog box if the fitting function string is
%                    not specified.
% 2005/12/12: v1.31, display a message after each fit
% 2005/12/13: v1.32, starts with the dialog box if the order for the
%                    polynomial fit is not specified.
% 2005/12/20: v1.33, display the message box only if fp.selectfitmsgbox is 
%                    'on' in fitparam.
% 2006/01/28: v1.34, first check if a curve is present.
% 2006/02/08: v2.00, new syntax. now works with the fit structure F.
% 2006/02/10: v2.01, displays the legend.
% 2006/02/16: v2.02, use evalfit
% 2006/02/27: v2.03, bug fixed for lin/log.
% 2006/03/08: v2.04, idem v1.32
% 2006/09/04: v2.10, temporarilly hide the legend (drawpolygon is buggy
%                    when a legend is present).
%                    '$' and '!' accepted instead of ';'
% 2006/09/06: v2.11, bug fixed when f.hdata is an invalid handle


if exist('fitparam.m','file'),
    fp=fitparam;
else
    error('No fitparam file found.');
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input arguments:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==0,
    efroot=fileparts(mfilename('fullpath'));   % directory where the ezyfit toolbox is installed
    prevfitfile=[efroot filesep 'prevfit.mat'];
    if exist(prevfitfile,'file'),
        load(prevfitfile);
    else
        fun='m1 + m2 * x; m1 = 1; m2 = 1';
    end;
    fun=inputdlg('Enter a default fit, a user-defined fit or directly a fit equation',...
        'General Curve Fit Definition',1,{fun});
    if length(fun)
        fun=fun{1};
        fun=strrep(fun,'$',';');  % new v2.10
        fun=strrep(fun,'!',';');
        save(prevfitfile,'fun');
        % if the option 'lin' or 'log' is not specified, use the Y-scale of
        % the current figure:
        if (isempty(findstr(strrep(fun,' ',''),';lin'))) && (isempty(findstr(strrep(fun,' ',''),';log'))),
            mode=get(gca,'YScale');
            fun=strrep([fun ';' mode(1:3)],';;',';');
        end;
        f=fit(fun); % fit the whole data
    else
        return;
    end;
elseif nargin>=1,
    if isstruct(varargin{1}), % if argument is a structure
        f=varargin{1};
    elseif ischar(varargin{1}), % if argument is a string
        fun=varargin{1};
        fun=strrep(fun,'$',';');  % new v2.10
        fun=strrep(fun,'!',';');
        if sum(strcmp(fun,{'nearest','spline','pchip','cubic','v5cubic'})),
            f.name=fun;
            [f.x, f.y, f.hdata]=pickdata;
        else  % if the string is not an interpolation  
            if strcmp(fun,'poly'), % new v2.04
                str_ord=inputdlg('Order of the polynomial fit','Polynomial order',1,{'2'});
                if length(str_ord),
                    fun=['poly' str_ord{1}];
                else
                    return;
                end;
            end;
            % if the option 'lin' or 'log' is not specified, use the Y-scale of
            % the current figure:
            if (isempty(findstr(strrep(fun,' ',''),';lin'))) && (isempty(findstr(strrep(fun,' ',''),';log'))),
                mode=get(gca,'YScale');
                fun=strrep([fun ';' mode(1:3)],';;',';');
            end;
            f.name=fun;
            [f.x, f.y, f.hdata]=pickdata;
        end;
    else
        error('Use selectfit with a string or a fit structure.');
    end;
end;


% output fit structure: idem as f, but with additional information.
fout=f;


if length(fp.fitcolor)==3,
    fitcolor=fp.fitcolor; % fixed color
else
    if ishandle(f.hdata)
        co=get(f.hdata); % object properties of the data
        if isfield(co,'Color'),
            fitcolor=max(0,min(1,co.Color*fp.fitcolor)); % color indexed from that of the data
        else
            % hist data do not have Color property
            fitcolor=[0 0 0];
        end;
        sizeselectpt=min(3,co.MarkerSize-1); % size of selected points, slightly smaller than data
    else
        fitcolor=[0 0 0];
        sizeselectpt=3;
    end;
end;


if strcmp(fp.selectfitmsgbox,'on'),
    % new v1.31, changed v1.33
    hmsg=annotation('textbox',[.2 .35 .4 .1],'BackgroundColor',[1 .95 .95],...
        'EdgeColor','red','Color','red','String',...
        {'Make a selection,','or click right mousebutton to exit.'});
    pause(1);
    delete(hmsg);
end;

if isempty(legend),  % new v2.10
    islegend = false;
else
    islegend = true;
    legend off;
end;


contfit=1;
while contfit,
    [xs,ys]=selectdata;
    if length(xs),   % non-empty plygon
        fout.xs=xs;
        fout.ys=ys;
        % deletes the previous fit (if any):
        if isfield(fout,'hfit'), delete(fout.hfit); end;
        if exist('hfit','var'), delete(hfit); end;
        if exist('hsel','var'), delete(hsel); end;
        if exist('heqbox','var'), delete(heqbox); end;
        switch f.name
            case {'nearest','spline','pchip','cubic','v5cubic'},

                % determines the X points used to compute and display the
                % fitted curve (an interpolation cannot be extrapolated!)
                if strcmp(get(gca,'XScale'),'log'),
                    xfit=logspace(log10(min(xs)),log10(max(xs)),fp.npt);
                else
                    xfit=linspace(min(xs),max(xs),fp.npt);
                end;

                yfit = interp1(xs,ys,xfit,fout.name);
                
                % makes the equation box:
                if strcmp(fp.dispeqboxmode,'on')
                    heqbox = showeqbox(fout,'transparent');
                end;
                
            otherwise
               
                fun = fout.name;
                if nargin<=1,
                    fout=fit(xs,ys, fun);
                else
                    fout=fit(xs,ys, fun, varargin{2:end});
                end;
                fout.hdata = f.hdata;

                % determines the X points used to compute and display the fitted curve
                switch(fp.extrapol),
                    case 'none',
                        xminfit=min(xs);  xmaxfit=max(xs);
                    case 'data'
                        xminfit=min(fout.x);  xmaxfit=max(fout.x);
                    case 'fig',
                        icg=get(gca);
                        xminfit=icg.XLim(1);  xmaxfit=icg.XLim(2);
                end;
                if strcmp(get(gca,'XScale'),'log'),
                    xfit=logspace(log10(xminfit),log10(xmaxfit),fp.npt);
                else
                    xfit=linspace(xminfit,xmaxfit,fp.npt);
                end;
                
                % evaluate the fit function:
                yfit = evalfit(fout, xfit);

                % makes the equation box:
                if strcmp(fp.dispeqboxmode,'on')
                    heqbox = showeqbox(fout,'transparent');
                end;
        end;

        % plots the interpolated / fitted curve:
        hold on;
        fout.hfit=plot(xfit,yfit,'Color',fitcolor,'LineStyle',fp.fitlinestyle,'LineWidth',fp.fitlinewidth);
        set(fout.hfit,'UserData', 'fit'); % this tag is useful to delete the fit later
        if length(fp.markerselectpt)
            hsel=plot(xs,ys,'Color',fitcolor,'Marker',fp.markerselectpt,'LineStyle','none','MarkerSize',sizeselectpt);
            set(hsel,'UserData','seldata');
        end;
        hold off;
        
        if strcmp(fp.selectfitmsgbox,'on'),
            % new v1.31, changed v1.33
            hmsg=annotation('textbox',[.2 .35 .4 .1],'BackgroundColor',[1 .95 .95],...
                'EdgeColor','red','Color','red','String',...
                {'Make another selection,','or click right mousebutton to confirm this one.'});
            pause(1);
            delete(hmsg);
        end;
        
    else  % empty polygon => end of fitting process
        contfit=0;
        % at the end of the fitting process the box is no more transparent:
        if exist('heqbox','var'),
            set(heqbox,'BackgroundColor','white');
        end;
    end;
end;


if isfield(fout,'hfit'); % if a fit has been done
    % output fit structure
    fout.heqbox = heqbox;
    delete(hsel);
    
    % give a 'DisplayName' to the fit:
    if length(fout.name)>10,
        fitname='fit';
    else
        fitname=greekize(fout.name);
    end;
    setautodisplayname; % give a name to all the data in the figure (if they don't have one yet)
    if ishandle(fout.hdata)
        fitname=[fitname ' (' get(fout.hdata,'DisplayName') ')'];
    end;
    set(fout.hfit,'DisplayName',fitname);

    if islegend
        legend show;   % put back the legend (new v2.10)
    end;
                
    % updates the legend:
    if strcmp(fp.dispfitlegend,'on')
        legend off;  % refresh the legend
        legend show;
    end;
    
    % opens the Array Editor with the fit coefficients:
    if (strcmp(fp.editcoeffmode,'on') && (isfield(fout,'m'))),
        editcoeff(fout);
    end;
    
    % displays the fit equation in the command window if no output
    % argument:
    if ~nargout
        if isfield(fout,'m'),
            dispeqfit(fout);
        end;
        clear fout;
    end;
else
    if islegend
        legend show;   % put back the legend (new v2.10)
    end;
    if ~nargout
        clear fout;
    else
        fout=[];
    end;
end;
